package com.fjhx.email.utils.email;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.sun.mail.imap.IMAPFolder;
import com.sun.mail.imap.IMAPMessage;
import com.sun.mail.imap.IMAPStore;

import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeUtility;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.*;

public class ImapUtil {

    private static final Properties properties;

    private static final Session session;

    private static final HashMap<String, String> storeId;

    static {

        properties = new Properties();

        // 开启ssl
        properties.put("mail.imap.ssl.enable", true);

        // 套接字连接超时值(毫秒)。此超时由java.net.Socket实现。默认是无限超时。
        properties.put("mail.imap.connectiontimeout", 1000 * 60);

        // 套接字读取超时值(毫秒)。此超时由java.net.Socket实现。默认是无限超时。
        properties.put("mail.imap.timeout", 1000 * 60 * 2);

        // 套接字写超时值，单位为毫秒。此超时是通过使用java.util.concurrent实现的。
        // ScheduledExecutorService每个连接，调度线程在超时过期时关闭套接字。
        // 因此，使用此超时的开销是每个连接一个线程。默认是无限超时。
        properties.put("mail.imap.writetimeout", 1000 * 60 * 2);

        session = Session.getInstance(properties);

        storeId = new HashMap<>(4);
        storeId.put("name", "fly");
        storeId.put("version", "1.0");
        storeId.put("vendor", "flyClient");
        storeId.put("support-email", "my@fly.com");

    }

    // public static void main(String[] args) throws Exception {
    //
    //     IMAPStore imapStore = getIMAPStore("imap.qq.com", "2428241269@qq.com", "kdypajdrwfhtdihb");
    //     // IMAPStore imapStore = getIMAPStore("imap.163.com", "18706065630@163.com", "CNZGQOIFETVQCZYV");
    //     // IMAPStore imapStore = getIMAPStore("imap.gmail.com", "y2428241269@gmail.com", "upoujrquwweqtsuk");
    //
    //     IMAPFolder folder = (IMAPFolder) imapStore.getFolder("INBOX");
    //     folder.open(Folder.READ_ONLY);
    //     Message[] messages = folder.getMessages();
    //
    //     Message message = messages[messages.length - 4];
    //
    //     setTextAndAttachment(message, new TextAndAttachment() {
    //
    //         @Override
    //         public void setPlain(String plain) {
    //
    //         }
    //
    //         @Override
    //         public void setHtml(String html) {
    //             // System.out.println(html);
    //         }
    //
    //         @Override
    //         public void setAttachment(String fileName, InputStream is) {
    //
    //             System.out.println("setAttachment");
    //
    //             FileUtil.writeFromStream(is, "E:\\" + fileName);
    //
    //         }
    //
    //         @Override
    //         public void unknown(Part part) throws MessagingException {
    //
    //             System.out.println("This is an unknown type：" + part.getContentType());
    //         }
    //
    //     });
    //
    //     System.out.println("end");
    //
    // }

    /**
     * @param host     连接地址
     * @param user     用户名
     * @param password 密码
     * @return 连接信息
     */
    public static IMAPStore getStore(String host, String user, String password) throws MessagingException {
        IMAPStore store = (IMAPStore) session.getStore("imap");
        store.connect(host, user, password);
        store.id(storeId);
        return store;
    }

    /**
     * 获取邮箱文件夹
     *
     * @param imapStore 连接信息
     * @return 文件夹列表
     */
    public static List<String> getFolderNameList(IMAPStore imapStore) throws MessagingException {
        IMAPFolder folder = (IMAPFolder) imapStore.getDefaultFolder();
        Folder[] folders = folder.list();
        List<String> mailFolderList = new ArrayList<>();
        addMailFolderList(mailFolderList, folders);
        return mailFolderList;
    }

    /**
     * 设置邮件已读
     *
     * @param message 邮件
     */
    public static void setSeen(Message message) throws MessagingException {
        Flags flags = message.getFlags();
        if (!flags.contains(Flags.Flag.SEEN)) {
            flags.add(Flags.Flag.SEEN);
            message.setFlags(flags, true);
        }
    }

    /**
     * 获取标签
     */
    public static String getFlags(Flags flags) {
        if (flags == null) {
            return StringPool.EMPTY;
        }

        Flags.Flag[] systemFlags = flags.getSystemFlags();
        if (systemFlags.length == 0) {
            return StringPool.EMPTY;
        }

        StringJoiner stringJoiner = new StringJoiner(",");
        for (Flags.Flag systemFlag : systemFlags) {
            stringJoiner.add(MailFlagEnum.getType(systemFlag));
        }
        return stringJoiner.toString();
    }

    /**
     * 获取邮件 收件人、抄送人、密送人、回复人
     *
     * @param message 邮件
     * @return 邮件 收件人、抄送人、密送人、回复人
     */
    public static List<MessageAddress> getMessageAddressList(Message message) throws MessagingException {
        Address[] to = message.getRecipients(Message.RecipientType.TO);
        Address[] cc = message.getRecipients(Message.RecipientType.CC);
        Address[] bcc = message.getRecipients(Message.RecipientType.BCC);
        Address[] replyTo = message.getReplyTo();

        List<MessageAddress> mailAddressList = new ArrayList<>();
        addMailAddressList(mailAddressList, to, AddressTypeEnum.TO.getKey());
        addMailAddressList(mailAddressList, cc, AddressTypeEnum.CC.getKey());
        addMailAddressList(mailAddressList, bcc, AddressTypeEnum.BCC.getKey());
        addMailAddressList(mailAddressList, replyTo, AddressTypeEnum.REPLY_TO.getKey());
        return mailAddressList;
    }

    /**
     * 赋值邮件基本信息
     */
    public static void setInfo(IMAPFolder folder, IMAPMessage message, MessageInfo messageInfo) throws MessagingException {
        messageInfo.setUid(folder.getUID(message));
        messageInfo.setSubject(message.getSubject());
        messageInfo.setSentDate(message.getSentDate());
        messageInfo.setReceivedDate(message.getReceivedDate());
        InternetAddress sender = (InternetAddress) message.getSender();
        if (sender != null) {
            messageInfo.setSenderAddress(sender.getAddress());
            messageInfo.setSenderPersonal(sender.getPersonal());
        }
    }

    /**
     * 赋值邮件正文以及附件
     */
    public static void setTextAndAttachment(Part part, TextAndAttachment textAndAttachment) throws MessagingException, IOException {
        if (part.isMimeType("text/plain")) {
            textAndAttachment.setPlain((String) part.getContent());
        } else if (part.isMimeType("text/html")) {
            textAndAttachment.setHtml((String) part.getContent());
        } else if (part.isMimeType("multipart/*")) {
            Multipart mp = (Multipart) part.getContent();
            for (int i = 0; i < mp.getCount(); i++) {
                setTextAndAttachment(mp.getBodyPart(i), textAndAttachment);
            }
        } else if (part.isMimeType("message/rfc822")) {
            setTextAndAttachment((Part) part.getContent(), textAndAttachment);
        } else {
            Object obj = part.getContent();
            if (obj instanceof InputStream) {
                String fileName = decodeText(part.getFileName());
                textAndAttachment.setAttachment(fileName, (InputStream) obj);
            } else {
                textAndAttachment.unknown(part);
            }
        }
    }

    /**
     * 关闭文件夹连接
     *
     * @param folder 文件夹信息
     */
    public static void closeFolder(IMAPFolder folder) throws MessagingException {
        if (folder == null) {
            return;
        }
        if (folder.isOpen()) {
            folder.close();
        }
    }

    /**
     * 关闭邮箱连接
     *
     * @param store 邮件连接
     */
    public static void closeStore(IMAPStore store) throws MessagingException {
        if (store == null) {
            return;
        }
        if (store.isConnected()) {
            store.close();
        }
    }

    /**
     * 文本解码
     */
    private static String decodeText(String encodeText) throws UnsupportedEncodingException {
        if (StrUtil.isBlank(encodeText)) {
            return StrUtil.EMPTY;
        } else {
            return MimeUtility.decodeText(encodeText);
        }
    }

    /**
     * 添加文件夹名称
     */
    private static void addMailFolderList(List<String> mailFolderList, Folder[] folders) throws MessagingException {
        for (Folder folder : folders) {
            if (folder.getType() == Folder.HOLDS_FOLDERS) {
                addMailFolderList(mailFolderList, folder.list());
            } else {
                mailFolderList.add(folder.getFullName());
            }
        }
    }

    /**
     * 添加邮件地址
     */
    private static void addMailAddressList(List<MessageAddress> mailAddressList, Address[] addresses, int type) {
        if (addresses == null) {
            return;
        }
        for (Address address : addresses) {
            InternetAddress internetAddress = (InternetAddress) address;

            MessageAddress mailAddress = new MessageAddress();
            mailAddress.setType(type);
            mailAddress.setEmail(internetAddress.getAddress());
            mailAddress.setName(internetAddress.getPersonal());
            mailAddressList.add(mailAddress);
        }
    }

}
