package com.fjhx.email.utils.email;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fjhx.email.entity.dto.ObsFile;
import com.fjhx.email.entity.dto.SendDto;
import com.sun.mail.smtp.SMTPMessage;

import javax.activation.DataHandler;
import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Properties;

public class SmtpUtil {

    // public static void main(String[] args) throws Exception {
    //     SendDto.FileInfo attachment = new SendDto.FileInfo();
    //     attachment.setFileUrl("https://os.winfaster.cn/fjhx/saas/GrandStar/2023/03/08/png/3092592f2d614d83854fad5e4c107143.png");
    //     attachment.setFileName("board-1.png");
    //
    //     SendDto.Address address = new SendDto.Address();
    //     address.setAddress("18706065630@163.com");
    //
    //     SendDto.Address bcc = new SendDto.Address();
    //     bcc.setAddress("2428241269@qq.com");
    //
    //     SendDto sendDto = new SendDto();
    //     sendDto.setTo(Collections.singletonList(address));
    //     sendDto.setBcc(Collections.singletonList(bcc));
    //     sendDto.setFileInfoList(Collections.singletonList(attachment));
    //     sendDto.setSubject("测试标题");
    //     sendDto.setContent("测试内容");
    //
    //     sendMail("imap.qq.com", "2428241269@qq.com", "kdypajdrwfhtdihb", sendDto);
    // }

    /**
     * 发送邮件
     */
    public static void sendMail(String host, String user, String password, SendDto sendDto) throws Exception {

        // 创建一个配置文件并保存
        Properties properties = new Properties();
        properties.put("mail.smtp.host", host);
        properties.put("mail.smtp.auth", true);
        properties.put("mail.smtp.ssl.enable", true);

        // auth认证
        Authenticator authenticator = new Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(user, password);
            }
        };

        // 创建一个session对象
        Session session = Session.getInstance(properties, authenticator);

        // 创建邮件对象
        SMTPMessage mimeMessage = new SMTPMessage(session);

        // 邮件发送人
        mimeMessage.setFrom(new InternetAddress(user));

        List<SendDto.Address> to = sendDto.getTo();
        if (ObjectUtil.isNotEmpty(to)) {
            mimeMessage.setRecipients(Message.RecipientType.TO, getAddresses(to));
        }

        List<SendDto.Address> cc = sendDto.getCc();
        if (ObjectUtil.isNotEmpty(cc)) {
            mimeMessage.setRecipients(Message.RecipientType.CC, getAddresses(cc));
        }

        List<SendDto.Address> bcc = sendDto.getBcc();
        if (ObjectUtil.isNotEmpty(bcc)) {
            mimeMessage.setRecipients(Message.RecipientType.BCC, getAddresses(bcc));
        }

        List<SendDto.Address> replyTo = sendDto.getReplyTo();
        if (ObjectUtil.isNotEmpty(replyTo)) {
            mimeMessage.setReplyTo(getAddresses(replyTo));
        }

        // 邮件标题
        mimeMessage.setSubject(sendDto.getSubject());

        // 邮件内容大对象
        MimeMultipart mimeMultipart = new MimeMultipart();

        // 对象内添加正文
        MimeBodyPart content = new MimeBodyPart();
        content.setContent(sendDto.getContent(), "text/html; charset=utf-8");
        mimeMultipart.addBodyPart(content);

        // 对象内添加邮件
        List<ObsFile> fileInfoList = sendDto.getFileList();
        if (ObjectUtil.isNotEmpty(fileInfoList)) {
            for (ObsFile attachment : fileInfoList) {
                DataHandler dataHandler = new DataHandler(new URL(attachment.getFileUrl()));
                MimeBodyPart mimeBodyPart = new MimeBodyPart();
                mimeBodyPart.setDataHandler(dataHandler);
                mimeBodyPart.setContentID(IdWorker.getIdStr());
                mimeBodyPart.setFileName(MimeUtility.encodeText(attachment.getFileName()));
                mimeMultipart.addBodyPart(mimeBodyPart);
            }
        }

        // 添加邮件内容
        mimeMessage.setContent(mimeMultipart);

        // 发送
        Transport.send(mimeMessage);

    }


    private static InternetAddress[] getAddresses(List<SendDto.Address> addressList) throws UnsupportedEncodingException {
        InternetAddress[] internetAddresses = new InternetAddress[addressList.size()];
        for (int i = 0; i < addressList.size(); i++) {
            InternetAddress internetAddress = new InternetAddress();
            internetAddress.setPersonal(addressList.get(i).getPersonal(), StandardCharsets.UTF_8.name());
            internetAddress.setAddress(addressList.get(i).getAddress());
            internetAddresses[i] = internetAddress;
        }
        return internetAddresses;
    }

}
